from redpepper.operations import Operation, Result


class Exec(Operation):
    def __init__(self, script, env=None):
        self._script = script
        self.script = compile(script, "<py.Exec>", "exec")
        self.env = globals().copy()
        if env:
            self.env.update(env)

    def __str__(self):
        cmd = self._script
        if "\n" in cmd:
            cmd = cmd.split("\n", 1)[0] + "..."
        if len(cmd) > 50:
            cmd = cmd[:50] + "..."
        return f'py.Exec("{cmd.strip()}")'

    def ensure(self, agent):
        result = Result(self)
        try:
            exec(self.script, self.env, locals())
        except Exception:
            result.exception()
        return result



import subprocess

from redpepper.operations import Operation, Result


class Running(Operation):
    _no_changes_text = "The service is already running."

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return f"systemd.Running({self.name})"

    def test(self, agent):
        cmd = ["systemctl", "is-active", self.name]
        p = subprocess.run(cmd, stdout=subprocess.DEVNULL)
        return p.returncode == 0

    def run(self, agent):
        result = Result(self)
        cmd = ["systemctl", "start", self.name]
        p = subprocess.run(cmd, capture_output=True, text=True)
        if result.check_completed_process(p).succeeded:
            result += f"Service {self.name} started."
            result.changed = True
        return result


class Enabled(Operation):
    _no_changes_text = "The service is already enabled."

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return f"systemd.Enabled({self.name})"

    def test(self, agent):
        cmd = ["systemctl", "is-enabled", self.name]
        p = subprocess.run(cmd, stdout=subprocess.DEVNULL)
        return p.returncode == 0

    def run(self, agent):
        result = Result(self)
        cmd = ["systemctl", "enable", self.name]
        p = subprocess.run(cmd, capture_output=True, text=True)
        if result.check_completed_process(p).succeeded:
            result += f"Service {self.name} enabled."
            result.changed = True
        return result


class Restart(Operation):
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return f"systemd.Restart({self.name})"

    def run(self, agent):
        result = Result(self)
        cmd = ["systemctl", "restart", self.name]
        p = subprocess.run(cmd, capture_output=True, text=True)
        if result.check_completed_process(p).succeeded:
            result += f"Service {self.name} restarted."
            result.changed = True
        return result


class Reload(Operation):
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return f"systemd.Reload({self.name})"

    def run(self, agent):
        result = Result(self)
        cmd = ["systemctl", "reload", self.name]
        p = subprocess.run(cmd, capture_output=True, text=True)
        if result.check_completed_process(p).succeeded:
            result += f"Service {self.name} reloaded."
            result.changed = True
        return result



from redpepper.operations import Operation, Result


class Noop(Operation):
    def __init__(self):
        pass

    def __str__(self):
        return "no-op"

    def ensure(self, agent):
        result = Result(self)
        result.succeeded = True
        return result



import subprocess
import sys

if sys.platform == "linux":
    DISTROS = {
        "debian": "apt",
        "ubuntu": "apt",
        "linuxmint": "apt",
    }
    d = subprocess.run(
        ["lsb_release", "-is"],
        stdout=subprocess.PIPE,
        stderr=subprocess.DEVNULL,
        text=True,
    ).stdout.strip()
    package_manager = DISTROS.get(d.lower())
    if package_manager == "apt":
        from redpepper.operations.apt import Installed  # noqa: F401
    else:
        raise ImportError(f"Unknown Linux distro for package module: {d})")
else:
    raise ImportError(f"Unsupported platform for package module: {sys.platform}")



import os

from redpepper.common.operations import Operation, Result

__all__ = [
    "Operation",
    "Result",
]

# Set here so it can be changed
custom_operations_dir = os.path.join(os.path.dirname(__file__))



from redpepper.operations import Operation, Result


class Request(Operation):
    def __init__(self, request_name, **kwargs):
        self.request_name = request_name
        self.kwargs = kwargs

    def __str__(self):
        return f"request.Request({self.request_name})"

    def run(self, agent):
        result = Result(self)
        data = agent.request(self.request_name, **self.kwargs)
        result.succeeded = True
        result += str(data)
        return result


class StateRequest(Operation):
    def __init__(self, request_name, **kwargs):
        self.request_name = request_name
        self.kwargs = kwargs

    def __str__(self):
        return f"request.StateRequest({self.request_name})"

    def run(self, agent):
        result = Result(self)
        data = agent.request(self.request_name, **self.kwargs)
        result.succeeded = data["succeeded"]
        result.changed = data["changed"]
        result += data["output"]
        return result
